@@ -5,11 +5,12 @@ import typing as t
 from contextlib import contextmanager
 from pathlib import Path
 
+# isort: off
 try:
-    from contextlib import chdir as chdir_cm  # type: ignore[attr-defined]
+    from contextlib import chdir as chdir_cm  # type: ignore[attr-defined, unused-ignore] # noqa: E501
 except ImportError:
 
-    @contextmanager  # type: ignore[no-redef]
+    @contextmanager  # type: ignore[no-redef, unused-ignore]
     def chdir_cm(path: os.PathLike) -> t.Iterator[None]:
         """Temporarily change the current directory, recovering on exit."""
         original_wd = Path.cwd()
@@ -20,4 +21,13 @@ except ImportError:
             os.chdir(original_wd)
 
 
-__all__ = ("chdir_cm",)  # noqa: WPS410
+# isort: on
+
+
+try:
+    from tomllib import loads as load_toml_from_string
+except ImportError:
+    from tomli import loads as load_toml_from_string
+
+
+__all__ = ("chdir_cm", "load_toml_from_string")  # noqa: WPS410
