@@ -15,6 +15,26 @@ def test_build_simple():
     assert str(u) == "http://127.0.0.1"
 
 
+def test_url_build_ipv6():
+    u = URL.build(scheme="http", host="::1")
+    assert str(u) == "http://[::1]"
+
+
+def test_url_build_ipv6_brackets_encoded():
+    u = URL.build(scheme="http", host="[::1]", encoded=True)
+    assert str(u) == "http://[::1]"
+
+
+def test_url_build_ipv6_brackets_not_encoded():
+    u = URL.build(scheme="http", host="::1", encoded=False)
+    assert str(u) == "http://[::1]"
+
+
+def test_url_ipv4_in_ipv6():
+    u = URL.build(scheme="http", host="2001:db8:122:344::192.0.2.33")
+    assert str(u) == "http://[2001:db8:122:344::c000:221]"
+
+
 def test_build_with_scheme():
     u = URL.build(scheme="blob", path="path")
     assert str(u) == "blob:path"
@@ -100,6 +120,25 @@ def test_build_with_authority_and_host():
         URL.build(authority="host.com", host="example.com")
 
 
+@pytest.mark.parametrize(
+    ("host", "is_authority"),
+    [
+        ("user:pass@host.com", True),
+        ("user@host.com", True),
+        ("host:com", False),
+        ("not_percent_encoded%Zf", False),
+        ("still_not_percent_encoded%fZ", False),
+        *(("other_gen_delim_" + c, False) for c in "/?#[]"),
+    ],
+)
+def test_build_with_invalid_host(host: str, is_authority: bool):
+    match = r"Host '[^']+' cannot contain '[^']+' \(at position \d+\)"
+    if is_authority:
+        match += ", if .* use 'authority' instead of 'host'"
+    with pytest.raises(ValueError, match=f"{match}$"):
+        URL.build(host=host)
+
+
 def test_build_with_authority():
     url = URL.build(scheme="http", authority="степан:bar@host.com:8000", path="path")
     assert (
@@ -114,6 +153,31 @@ def test_build_with_authority_without_encoding():
     assert str(url) == "http://foo:bar@host.com:8000/path"
 
 
+def test_build_with_authority_empty_host_no_scheme():
+    url = URL.build(authority="", path="path")
+    assert str(url) == "path"
+
+
+def test_build_with_authority_and_only_user():
+    url = URL.build(scheme="https", authority="user:@foo.com", path="/path")
+    assert str(url) == "https://user:@foo.com/path"
+
+
+def test_build_with_authority_with_port():
+    url = URL.build(scheme="https", authority="foo.com:8080", path="/path")
+    assert str(url) == "https://foo.com:8080/path"
+
+
+def test_build_with_authority_with_ipv6():
+    url = URL.build(scheme="https", authority="[::1]", path="/path")
+    assert str(url) == "https://[::1]/path"
+
+
+def test_build_with_authority_with_ipv6_and_port():
+    url = URL.build(scheme="https", authority="[::1]:81", path="/path")
+    assert str(url) == "https://[::1]:81/path"
+
+
 def test_query_str():
     u = URL.build(scheme="http", host="127.0.0.1", path="/", query_string="arg=value1")
     assert str(u) == "http://127.0.0.1/?arg=value1"
